@@ -259,22 +259,22 @@ bamf_application_dispose (GObject *object)
 {
   BamfApplication *self;
   BamfApplicationPrivate *priv;
-  
+
   self = BAMF_APPLICATION (object);
   priv = self->priv;
-  
+
   if (priv->application_type)
     {
       g_free (priv->application_type);
       priv->application_type = NULL;
     }
-  
+
   if (priv->desktop_file)
     {
       g_free (priv->desktop_file);
       priv->desktop_file = NULL;
-    }  
-  
+    }
+
   if (priv->proxy)
     {
       dbus_g_proxy_disconnect_signal (priv->proxy,
@@ -311,6 +311,7 @@ bamf_application_set_path (BamfView *view, const char *path)
   if (priv->proxy == NULL)
     {
       g_critical ("Unable to get org.ayatana.bamf.application application");
+      return;
     }
 
   dbus_g_proxy_add_signal (priv->proxy,
@@ -461,7 +462,7 @@ bamf_application_new_favorite (const char * favorite_path)
   BamfApplication *self;
   GKeyFile        *desktop_keyfile;
   GKeyFileFlags    flags;
-  const gchar     *type;
+  gchar           *type;
   gboolean         supported = FALSE;
   
   // check that we support this kind of desktop file
@@ -474,6 +475,7 @@ bamf_application_new_favorite (const char * favorite_path)
         supported = TRUE;
 
       g_key_file_free (desktop_keyfile);
+      g_free (type);
     }
   if (!supported)
     return NULL;
