@@ -96,6 +96,7 @@ struct _BamfViewPrivate
   guint            set_flags;
   gboolean         is_closed;
   gboolean         sticky;
+  GList           *cached_children;
 };
 
 static void
@@ -149,7 +150,7 @@ bamf_view_get_children (BamfView *view)
   BamfViewPrivate *priv;
 
   g_return_val_if_fail (BAMF_IS_VIEW (view), NULL);
-  
+
   if (BAMF_VIEW_GET_CLASS (view)->get_children)
     return BAMF_VIEW_GET_CLASS (view)->get_children (view);
 
@@ -158,6 +159,9 @@ bamf_view_get_children (BamfView *view)
 
   priv = view->priv;
 
+  if (priv->cached_children)
+    return g_list_copy (priv->cached_children);
+
   if (!dbus_g_proxy_call (priv->proxy,
                           "Children",
                           &error,
@@ -166,6 +170,7 @@ bamf_view_get_children (BamfView *view)
                           G_TYPE_INVALID))
     {
       g_warning ("Unable to fetch children: %s\n", error->message);
+      g_error_free (error);
       return NULL;
     }
 
@@ -174,13 +179,14 @@ bamf_view_get_children (BamfView *view)
 
   len = g_strv_length (children);
 
-  for (i = 0; i < len; i++)
+  for (i = len-1; i >= 0; i--)
     {
-      BamfView *view = bamf_factory_view_for_path (bamf_factory_get_default (), children[i]);
-      results = g_list_prepend (results, view);
+      view = bamf_factory_view_for_path (bamf_factory_get_default (), children[i]);
+      results = g_list_prepend (results, g_object_ref (view));
     }
 
-  return results;
+  priv->cached_children = results;
+  return g_list_copy (priv->cached_children);
 }
 
 static gboolean
@@ -192,7 +198,7 @@ bamf_view_get_boolean (BamfView *self, const char *method_name, guint flag)
 
   g_return_val_if_fail (BAMF_IS_VIEW (self), FALSE);
   priv = self->priv;
-  
+
   if (bamf_view_flag_is_set (self, flag))
     return bamf_view_get_flag (self, flag);
 
@@ -208,10 +214,10 @@ bamf_view_get_boolean (BamfView *self, const char *method_name, guint flag)
     {
       g_warning ("Failed to fetch boolean: %s", error->message);
       g_error_free (error);
-      
+
       return FALSE;
     }
-  
+
   bamf_view_set_flag (self, flag, result);
   return result;
 }
@@ -220,7 +226,7 @@ gboolean
 bamf_view_is_closed (BamfView *view)
 {
   g_return_val_if_fail (BAMF_IS_VIEW (view), TRUE);
-  
+
   return view->priv->is_closed;
 }
 
@@ -228,7 +234,7 @@ gboolean
 bamf_view_is_active (BamfView *view)
 {
   g_return_val_if_fail (BAMF_IS_VIEW (view), FALSE);
-  
+
   if (BAMF_VIEW_GET_CLASS (view)->is_active)
     return BAMF_VIEW_GET_CLASS (view)->is_active (view);
 
@@ -275,15 +281,33 @@ bamf_view_set_name (BamfView *view, const char *name)
   if (!g_strcmp0 (name, view->priv->local_name))
     return;
 
-  view->priv->local_name = g_strdup (name);
+  g_free (view->priv->local_name);
+
+  if (name && name[0] == '\0')
+    {
+      view->priv->local_name = NULL;
+    }
+  else
+    {
+      view->priv->local_name = g_strdup (name);
+    }
 }
 
 void
 bamf_view_set_icon (BamfView *view, const char *icon)
 {
   g_return_if_fail (BAMF_IS_VIEW (view));
-  
-  view->priv->local_icon = g_strdup (icon);
+
+  g_free (view->priv->local_icon);
+
+  if (icon && icon[0] == '\0')
+    {
+      view->priv->local_icon = NULL;
+    }
+  else
+    {
+      view->priv->local_icon = g_strdup (icon);
+    }
 }
 
 gboolean 
@@ -339,6 +363,12 @@ bamf_view_get_icon (BamfView *self)
       return NULL;
     }
 
+  if (icon && icon[0] == '\0')
+    {
+      g_free (icon);
+      return NULL;
+    }
+
   return icon;
 }
 
@@ -371,6 +401,12 @@ bamf_view_get_name (BamfView *self)
       return NULL;
     }
 
+  if (name && name[0] == '\0')
+    {
+      g_free (name);
+      return NULL;
+    }
+
   return name;
 }
 
@@ -427,9 +463,17 @@ static void
 bamf_view_on_child_added (DBusGProxy *proxy, char *path, BamfView *self)
 {
   BamfView *view;
+  BamfViewPrivate *priv;
 
   view = bamf_factory_view_for_path (bamf_factory_get_default (), path);
-  
+  priv = self->priv;
+
+  if (priv->cached_children)
+    {
+      g_list_free_full (priv->cached_children, g_object_unref);
+      priv->cached_children = NULL;
+    }
+
   g_signal_emit (G_OBJECT (self), view_signals[CHILD_ADDED], 0, view);
 }
 
@@ -437,8 +481,15 @@ static void
 bamf_view_on_child_removed (DBusGProxy *proxy, char *path, BamfView *self)
 {
   BamfView *view;
-
+  BamfViewPrivate *priv;
   view = bamf_factory_view_for_path (bamf_factory_get_default (), path);
+  priv = self->priv;
+
+  if (priv->cached_children)
+    {
+      g_list_free_full (priv->cached_children, g_object_unref);
+      priv->cached_children = NULL;
+    }
 
   g_signal_emit (G_OBJECT (self), view_signals[CHILD_REMOVED], 0, view);
 }
@@ -458,6 +509,7 @@ bamf_view_on_name_changed (DBusGProxy*  proxy,
                            const gchar* new_name,
                            BamfView*    self)
 {
+  g_free (self->priv->local_name);
   self->priv->local_name = g_strdup (new_name);
 
   g_signal_emit (self, view_signals[NAME_CHANGED], 0, old_name, new_name);
@@ -494,11 +546,17 @@ static void
 bamf_view_on_closed (DBusGProxy *proxy, BamfView *self)
 {
   BamfViewPrivate *priv;
-  
+
   priv = self->priv;
 
   priv->is_closed = TRUE;
-  
+
+  if (priv->cached_children)
+    {
+      g_list_free_full (priv->cached_children, g_object_unref);
+      priv->cached_children = NULL;
+    }
+
   if (priv->sticky && priv->proxy)
     {
       dbus_g_proxy_disconnect_signal (priv->proxy,
@@ -606,20 +664,37 @@ bamf_view_dispose (GObject *object)
   view = BAMF_VIEW (object);
 
   priv = view->priv;
-  
+
   if (priv->path)
     {
       g_free (priv->path);
       priv->path = NULL;
     }
-  
-    
+
   if (priv->type)
     {
       g_free (priv->type);
       priv->type = NULL;
     }
 
+  if (priv->local_icon)
+    {
+      g_free (priv->local_icon);
+      priv->local_icon = NULL;
+    }
+
+  if (priv->local_name)
+    {
+      g_free (priv->local_name);
+      priv->local_name = NULL;
+    }
+
+  if (priv->cached_children)
+    {
+      g_list_free_full (priv->cached_children, g_object_unref);
+      priv->cached_children = NULL;
+    }
+
   if (priv->proxy)
     {
       dbus_g_proxy_disconnect_signal (priv->proxy,
