@@ -21,6 +21,7 @@
 
 #include "bamf-legacy-window.h"
 #include "bamf-legacy-screen.h"
+#include "bamf-xutils.h"
 #include <libgtop-2.0/glibtop.h>
 #include <glibtop/procargs.h>
 #include <glibtop/procuid.h>
@@ -34,6 +35,7 @@ enum
 {
   NAME_CHANGED,
   STATE_CHANGED,
+  GEOMETRY_CHANGED,
   CLOSED,
 
   LAST_SIGNAL,
@@ -48,6 +50,7 @@ struct _BamfLegacyWindowPrivate
   gulong       closed_id;
   gulong       name_changed_id;
   gulong       state_changed_id;
+  gulong       geometry_changed_id;
   gboolean     is_closed;
 };
 
@@ -94,7 +97,6 @@ bamf_legacy_window_is_skip_tasklist (BamfLegacyWindow *self)
 {
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), FALSE);
 
-
   if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->is_skip_tasklist)
     return BAMF_LEGACY_WINDOW_GET_CLASS (self)->is_skip_tasklist (self);
 
@@ -215,6 +217,9 @@ bamf_legacy_window_save_mini_icon (BamfLegacyWindow *self)
   
   window = self->priv->legacy_window;
   
+  if (!window)
+    return NULL;
+  
   if (wnck_window_get_icon_is_fallback (window))
     return NULL;
   
@@ -235,7 +240,6 @@ bamf_legacy_window_get_pid (BamfLegacyWindow *self)
 {
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), 0);
 
-
   if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_pid)
     return BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_pid (self);
 
@@ -259,7 +263,7 @@ bamf_legacy_window_get_xid (BamfLegacyWindow *self)
   return (guint32) wnck_window_get_xid (self->priv->legacy_window);
 }
 
-BamfLegacyWindow * 
+BamfLegacyWindow *
 bamf_legacy_window_get_transient (BamfLegacyWindow *self)
 {
   BamfLegacyScreen *screen;
@@ -291,6 +295,19 @@ bamf_legacy_window_get_transient (BamfLegacyWindow *self)
   return NULL;
 }
 
+gint
+bamf_legacy_window_get_stacking_position (BamfLegacyWindow *self)
+{
+  BamfLegacyScreen *screen;
+  
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), -1);
+
+  screen = bamf_legacy_screen_get_default ();
+  g_return_val_if_fail (BAMF_IS_LEGACY_SCREEN (screen), -1);
+  
+  return g_list_index (bamf_legacy_screen_get_windows (screen), self);
+}
+
 static void
 handle_name_changed (WnckWindow *window, BamfLegacyWindow *self)
 {
@@ -310,6 +327,14 @@ handle_state_changed (WnckWindow *window,
   g_signal_emit (self, legacy_window_signals[STATE_CHANGED], 0);
 }
 
+static void
+handle_geometry_changed (WnckWindow *window, BamfLegacyWindow *self)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
+
+  g_signal_emit (self, legacy_window_signals[GEOMETRY_CHANGED], 0);
+}
+
 gboolean 
 bamf_legacy_window_is_closed (BamfLegacyWindow *self)
 {
@@ -318,6 +343,76 @@ bamf_legacy_window_is_closed (BamfLegacyWindow *self)
   return self->priv->is_closed;
 }
 
+void
+bamf_legacy_window_get_geometry (BamfLegacyWindow *self, gint *x, gint *y,
+                                 gint *width, gint *height)
+{
+  *x = 0;
+  *y = 0;
+  *width = 0;
+  *height = 0;
+
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_app_id)
+    BAMF_LEGACY_WINDOW_GET_CLASS (self)->get_app_id (self);
+
+  if (!self->priv->legacy_window)
+    return;
+
+  wnck_window_get_geometry (self->priv->legacy_window, x, y, width, height);
+}
+
+BamfWindowMaximizationType
+bamf_legacy_window_maximized (BamfLegacyWindow *self)
+{
+  WnckWindowState window_state;
+  BamfWindowMaximizationType maximization_type;
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), BAMF_WINDOW_FLOATING);
+
+  if (BAMF_LEGACY_WINDOW_GET_CLASS (self)->maximized)
+    return BAMF_LEGACY_WINDOW_GET_CLASS (self)->maximized (self);
+
+  if (!self->priv->legacy_window)
+    return BAMF_WINDOW_FLOATING;
+
+  window_state = wnck_window_get_state (self->priv->legacy_window);
+
+  gboolean vertical = (window_state & WNCK_WINDOW_STATE_MAXIMIZED_VERTICALLY);
+  gboolean horizontal = (window_state & WNCK_WINDOW_STATE_MAXIMIZED_HORIZONTALLY);
+
+  if (vertical && horizontal)
+    {
+      maximization_type = BAMF_WINDOW_MAXIMIZED;
+    }
+  else if (horizontal)
+    {
+      maximization_type = BAMF_WINDOW_HORIZONTAL_MAXIMIZED;
+    }
+  else if (vertical)
+    {
+      maximization_type = BAMF_WINDOW_VERTICAL_MAXIMIZED;
+    }
+  else
+    {
+      maximization_type = BAMF_WINDOW_FLOATING;
+    }
+
+  return maximization_type;
+}
+
+char *
+bamf_legacy_window_get_utf8_xprop (BamfLegacyWindow *self, const char* prop)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (self), NULL);
+
+  if (!self->priv->legacy_window)
+    return NULL;
+
+  guint xid = bamf_legacy_window_get_xid (self);
+  return bamf_xutils_get_window_hint (xid, prop, XInternAtom(gdk_x11_get_default_xdisplay (), "UTF8_STRING", False));
+}
+
 static void
 handle_window_closed (WnckScreen *screen,
                       WnckWindow *window,
@@ -325,15 +420,43 @@ handle_window_closed (WnckScreen *screen,
 {
   g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
   g_return_if_fail (WNCK_IS_WINDOW (window));
-  
-  self->priv->is_closed = TRUE;
 
-  if (window == self->priv->legacy_window)
+  if (self->priv->legacy_window == window)
     {
+      self->priv->is_closed = TRUE;
       g_signal_emit (self, legacy_window_signals[CLOSED], 0);
     }
 }
 
+static void
+handle_destroy_notify (gpointer *data, BamfLegacyWindow *self_was_here)
+{
+  BamfLegacyScreen *screen = bamf_legacy_screen_get_default ();
+  bamf_legacy_screen_inject_window (screen, GPOINTER_TO_UINT (data));
+}
+
+/* This utility function allows to set a BamfLegacyWindow as closed, notifying
+ * all its owners, and to reopen it once the current window has been destroyed.
+ * This allows to remap particular windows to different applications.         */
+void
+bamf_legacy_window_reopen (BamfLegacyWindow *self)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW (self));
+  g_return_if_fail (WNCK_IS_WINDOW (self->priv->legacy_window));
+
+  guint xid = bamf_legacy_window_get_xid (self);
+
+  /* Adding a weak ref to this object, causes to get notified after the object
+   * destruction, so once this BamfLegacyWindow has been closed and drestroyed
+   * the handle_destroy_notify() function will be called, and that will
+   * provide to iniject another window like this one to the BamfLegacyScreen  */
+  g_object_weak_ref (G_OBJECT (self), (GWeakNotify) handle_destroy_notify,
+                                                    GUINT_TO_POINTER (xid));
+
+  self->priv->is_closed = TRUE;
+  g_signal_emit (self, legacy_window_signals[CLOSED], 0);
+}
+
 static void
 bamf_legacy_window_dispose (GObject *object)
 {
@@ -362,6 +485,9 @@ bamf_legacy_window_dispose (GObject *object)
 
       g_signal_handler_disconnect (self->priv->legacy_window,
                                    self->priv->state_changed_id);
+
+      g_signal_handler_disconnect (self->priv->legacy_window,
+                                   self->priv->geometry_changed_id);
     }
 
   G_OBJECT_CLASS (bamf_legacy_window_parent_class)->dispose (object);
@@ -378,7 +504,7 @@ bamf_legacy_window_init (BamfLegacyWindow * self)
   screen = wnck_screen_get_default ();
 
   priv->closed_id = g_signal_connect (G_OBJECT (screen), "window-closed",
-    		                       (GCallback) handle_window_closed, self);
+                                      (GCallback) handle_window_closed, self);
 }
 
 static void
@@ -391,33 +517,40 @@ bamf_legacy_window_class_init (BamfLegacyWindowClass * klass)
   g_type_class_add_private (klass, sizeof (BamfLegacyWindowPrivate));
 
   legacy_window_signals [NAME_CHANGED] =
-  	g_signal_new ("name-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfLegacyWindowClass, name_changed),
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__VOID,
-  	              G_TYPE_NONE, 0);
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_NAME_CHANGED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, name_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 
   legacy_window_signals [STATE_CHANGED] =
-  	g_signal_new ("state-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfLegacyWindowClass, state_changed),
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__VOID,
-  	              G_TYPE_NONE, 0);
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_STATE_CHANGED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, state_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  legacy_window_signals [GEOMETRY_CHANGED] =
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_GEOMETRY_CHANGED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, geometry_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 
   legacy_window_signals [CLOSED] =
-  	g_signal_new ("closed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfLegacyWindowClass, closed),
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__VOID,
-  	              G_TYPE_NONE, 0);
-
-
+    g_signal_new (BAMF_LEGACY_WINDOW_SIGNAL_CLOSED,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (BamfLegacyWindowClass, closed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 BamfLegacyWindow *
@@ -429,10 +562,13 @@ bamf_legacy_window_new (WnckWindow *legacy_window)
   self->priv->legacy_window = legacy_window;
 
   self->priv->name_changed_id = g_signal_connect (G_OBJECT (legacy_window), "name-changed",
-    		                       (GCallback) handle_name_changed, self);
+                                                  (GCallback) handle_name_changed, self);
 
   self->priv->state_changed_id = g_signal_connect (G_OBJECT (legacy_window), "state-changed",
-                                       (GCallback) handle_state_changed, self);
+                                                   (GCallback) handle_state_changed, self);
+
+  self->priv->geometry_changed_id = g_signal_connect (G_OBJECT (legacy_window), "geometry-changed",
+                                                      (GCallback) handle_geometry_changed, self);
 
   return self;
 }
